@@ -1,9 +1,9 @@
-import math
-import statistics
-import thresholds
 from ocgis.calc.library.index import heat_index, duration
 from ocgis.calc.library.math import Convolve1D
 from ocgis.util.helpers import itersubclasses
+from . import math
+from . import statistics
+from . import thresholds
 
 
 class FunctionRegistry(dict):
@@ -16,7 +16,7 @@ class FunctionRegistry(dict):
         self.reg += [statistics.FrequencyPercentile, statistics.Mean, statistics.StandardDeviation, statistics.Max,
                      statistics.Median, statistics.Min, Convolve1D, statistics.MovingWindow, statistics.DailyPercentile]
         self.reg += [thresholds.Between, thresholds.Threshold]
-        self.reg += [heat_index.HeatIndex, duration.Duration, duration.FrequencyDuration]
+        self.reg += [heat_index.HeatIndex, duration.Duration]
 
         for cc in self.reg:
             self.update({cc.key: cc})
