@@ -1,17 +1,17 @@
 import datetime
 
-import netCDF4 as nc
+import six
 
 import ocgis
-from ocgis import constants
+from ocgis import RequestDataset
 from ocgis import env
-from ocgis.api.request.driver.vector import DriverVector
 from ocgis.calc.base import AbstractMultivariateFunction, AbstractKeyedOutputFunction
-from ocgis.calc.engine import OcgCalculationEngine
+from ocgis.calc.engine import CalculationEngine
 from ocgis.calc.eval_function import MultivariateEvalFunction
+from ocgis.constants import KeywordArgument
 from ocgis.conv.base import AbstractCollectionConverter
+from ocgis.driver.nc import DriverNetcdf, DriverNetcdfCF
 from ocgis.exc import DefinitionValidationError
-from ocgis.interface.base.crs import CFWGS84
 
 
 class NcConverter(AbstractCollectionConverter):
@@ -48,20 +48,21 @@ class NcConverter(AbstractCollectionConverter):
 
     @classmethod
     def validate_ops(cls, ops):
-        from ocgis.api.parms.definition import OutputFormat
+        from ocgis.ops.parms.definition import OutputFormat
 
         def _raise_(msg, ocg_arugument=OutputFormat):
             raise DefinitionValidationError(ocg_arugument, msg)
 
         # We can only write one request dataset to netCDF.
-        if len(ops.dataset) > 1 and ops.calc is None:
-            msg = ('Data packages (i.e. more than one RequestDataset) may not be written to netCDF. '
-                   'There are currently {dcount} RequestDatasets. Note, this is different than a '
-                   'multifile dataset.'.format(dcount=len(ops.dataset)))
+        len_ops_dataset = len(list(ops.dataset))
+        if len_ops_dataset > 1 and ops.calc is None:
+            msg = 'Data packages (i.e. more than one RequestDataset) may not be written to netCDF. There are ' \
+                  'currently {dcount} RequestDatasets. Note, this is different than a multifile dataset.'
+            msg = msg.format(dcount=len_ops_dataset)
             _raise_(msg, OutputFormat)
-        # We can write multivariate functions to netCDF however.
+        # We can write multivariate functions to netCDF.
         else:
-            if ops.calc is not None and len(ops.dataset) > 1:
+            if ops.calc is not None and len_ops_dataset > 1:
                 # Count the occurrences of these classes in the calculation list.
                 klasses_to_check = [AbstractMultivariateFunction, MultivariateEvalFunction]
                 multivariate_checks = []
@@ -86,26 +87,23 @@ class NcConverter(AbstractCollectionConverter):
         if ops.aggregate:
             msg = 'Data may not be aggregated for netCDF output. The aggregate parameter must be False.'
             _raise_(msg, OutputFormat)
-        # Either the input data CRS or WGS84 is required for data output.
-        if ops.output_crs is not None and not isinstance(ops.output_crs, CFWGS84):
-            msg = 'CFWGS84 is the only acceptable overloaded output CRS at this time for netCDF output.'
-            _raise_(msg, OutputFormat)
         # Calculations on raw values are not relevant as not aggregation can occur anyway.
         if ops.calc is not None:
             if ops.calc_raw:
                 msg = 'Calculations must be performed on original values (i.e. calc_raw=False) for netCDF output.'
                 _raise_(msg)
             # No keyed output functions to netCDF.
-            if OcgCalculationEngine._check_calculation_members_(ops.calc, AbstractKeyedOutputFunction):
+            if CalculationEngine._check_calculation_members_(ops.calc, AbstractKeyedOutputFunction):
                 msg = 'Keyed function output may not be written to netCDF.'
                 _raise_(msg)
 
     def _build_(self, coll):
-        ds = nc.Dataset(self.path, 'w', format=self._get_file_format_())
-        return ds
+        ret = {'path': self.path, 'dataset_kwargs': {'format': self._get_file_format_()},
+               'variable_kwargs': self._variable_kwargs, 'driver': DriverNetcdfCF}
+        return ret
 
     def _finalize_(self, ds):
-        ds.close()
+        pass
 
     def _get_file_format_(self):
         file_format = set()
@@ -118,16 +116,19 @@ class NcConverter(AbstractCollectionConverter):
                 ret = env.NETCDF_FILE_FORMAT
             else:
                 # If operations are available, check the request datasets and determine the best format for output.
-                for rd in self.ops.dataset.iter_request_datasets():
+                for rd in self.ops.dataset:
+                    # Only request dataset will have a possible output format.
+                    if not isinstance(rd, RequestDataset):
+                        continue
                     try:
-                        rr = rd.source_metadata['file_format']
+                        rr = rd.metadata['file_format']
                     except KeyError:
                         # Likely a shapefile request dataset which does not have an origin netcdf data format.
-                        if isinstance(rd.driver, DriverVector):
+                        if not isinstance(rd.driver, DriverNetcdf):
                             continue
                         else:
                             raise
-                    if isinstance(rr, basestring):
+                    if isinstance(rr, six.string_types):
                         tu = [rr]
                     else:
                         tu = rr
@@ -142,21 +143,33 @@ class NcConverter(AbstractCollectionConverter):
                         ret = env.NETCDF_FILE_FORMAT
         return ret
 
-    def _write_archetype_(self, arch, dataset, is_file_only, variable_kwargs):
+    def _write_archetype_(self, arch, write_kwargs, variable_kwargs):
         """
         Write a field to a netCDF dataset object.
 
         :param arch: The field to write.
-        :type arch: :class:`~ocgis.Field`
-        :param dataset: An open netCDF4 dataset object.
-        :type dataset: :class:`netCDF4.Dataset`
-        :param bool is_file_only: If ``True``, this is writing the template file only and there is no data fill.
+        :type arch: :class:`ocgis.new_interface.field.Field`
+        :param dict write_kwargs: Dictionary of parameters needed for the write.
         :param dict variable_kwargs: Optional keyword parameters to pass to the creation of netCDF4 variable objects.
          See http://unidata.github.io/netcdf4-python/#netCDF4.Variable.
         """
-        unlimited_to_fixedsize = self.options.get('unlimited_to_fixedsize', False)
-        arch.write_netcdf(dataset, file_only=is_file_only, unlimited_to_fixedsize=unlimited_to_fixedsize,
-                          **variable_kwargs)
+        # Append to the history attribute.
+        history_str = '\n{dt} UTC ocgis-{release}'.format(dt=datetime.datetime.utcnow(), release=ocgis.__release__)
+        if self.ops is not None:
+            history_str += ': {0}'.format(self.ops)
+        original_history_str = arch.attrs.get('history', '')
+        arch.attrs['history'] = original_history_str + history_str
+
+        # Pull in dataset and variable keyword arguments.
+        unlimited_to_fixedsize = self.options.get(KeywordArgument.UNLIMITED_TO_FIXED_SIZE, False)
+        write_kwargs[KeywordArgument.VARIABLE_KWARGS] = variable_kwargs
+        variable_kwargs[KeywordArgument.UNLIMITED_TO_FIXED_SIZE] = unlimited_to_fixedsize
+
+        # This is the output path. The driver handles MPI writing.
+        path = write_kwargs.pop(KeywordArgument.PATH)
+
+        # Write the field.
+        arch.write(path, **write_kwargs)
 
     def _write_coll_(self, ds, coll):
         """
@@ -169,61 +182,7 @@ class NcConverter(AbstractCollectionConverter):
         """
 
         # Get the target field from the collection.
-        arch = coll._archetype_field
+        arch = coll.archetype_field
         """:type arch: :class:`ocgis.Field`"""
 
-        # Get from operations if this is file only.
-        try:
-            is_file_only = self.ops.file_only
-        except AttributeError:
-            # No operations object available.
-            is_file_only = False
-
-        self._write_archetype_(arch, ds, is_file_only, self._variable_kwargs)
-
-        # Append to the history attribute.
-        history_str = '\n{dt} UTC ocgis-{release}'.format(dt=datetime.datetime.utcnow(), release=ocgis.__release__)
-        if self.ops is not None:
-            history_str += ': {0}'.format(self.ops)
-        original_history_str = ds.__dict__.get('history', '')
-        setattr(ds, 'history', original_history_str + history_str)
-
-
-class NcUgrid2DFlexibleMeshConverter(NcConverter):
-    @classmethod
-    def validate_ops(cls, ops):
-        from ocgis.api.parms.definition import OutputFormat
-
-        NcConverter.validate_ops(ops)
-        should_raise = False
-        if ops.abstraction == 'point':
-            should_raise = True
-        else:
-            for ds in ops.dataset.itervalues():
-                try:
-                    abstraction = ds.s_abstraction
-                except AttributeError:
-                    # Likely a field object.
-                    abstraction = ds.spatial.abstraction
-                if abstraction == 'point':
-                    should_raise = True
-                    break
-
-        if should_raise:
-            msg = 'Only polygons may be written to "{0}"'. \
-                format(constants.OUTPUT_FORMAT_NETCDF_UGRID_2D_FLEXIBLE_MESH, ops.abstraction)
-            raise DefinitionValidationError(OutputFormat, msg)
-
-    @staticmethod
-    def _write_archetype_(arch, dataset, is_file_only, variable_kwargs):
-        poly = arch.spatial.geom.polygon
-        """:type poly: :class:`ocgis.SpatialGeometryPolygonDimension`"""
-
-        try:
-            poly.write_to_netcdf_dataset_ugrid(dataset)
-        except AttributeError:
-            if poly is None:
-                msg = 'Field does not have a polygon representation.'
-                raise ValueError(msg)
-            else:
-                raise
+        self._write_archetype_(arch, ds, self._variable_kwargs)
