@@ -2,6 +2,8 @@
 
 import sys as _sys
 
+import six
+
 __all__ = ['Enum', 'IntEnum', 'unique']
 
 version = 1, 1, 2
@@ -23,17 +25,17 @@ except ImportError:
     OrderedDict = None
 
 try:
-    basestring
+    str
 except NameError:
     # In Python 2 basestring is the ancestor of both str and unicode
     # in Python 3 it's just str, but was missing in 3.1
-    basestring = str
+    str = str
 
 try:
-    unicode
+    str
 except NameError:
     # In Python 3 unicode no longer exists (it's just str)
-    unicode = str
+    str = str
 
 
 class _RouteClassAttributeToGetattr(object):
@@ -161,7 +163,7 @@ class EnumMeta(type):
         if type(classdict) is dict:
             original_dict = classdict
             classdict = _EnumDict()
-            for k, v in original_dict.items():
+            for k, v in list(original_dict.items()):
                 classdict[k] = v
 
         member_type, first_enum = metacls._get_mixins_(bases)
@@ -178,7 +180,7 @@ class EnumMeta(type):
         if __order__ is None:
             if pyver < 3.0:
                 try:
-                    __order__ = [name for (name, value) in sorted(members.items(), key=lambda item: item[1])]
+                    __order__ = [name for (name, value) in sorted(list(members.items()), key=lambda item: item[1])]
                 except TypeError:
                     __order__ = [name for name in sorted(members.keys())]
             else:
@@ -239,7 +241,7 @@ class EnumMeta(type):
             enum_member.__init__(*args)
             # If another member with the same value was already defined, the
             # new member becomes an alias to the existing one.
-            for name, canonical_member in enum_class._member_map_.items():
+            for name, canonical_member in list(enum_class._member_map_.items()):
                 if canonical_member.value == enum_member._value_:
                     enum_member = canonical_member
                     break
@@ -426,7 +428,7 @@ class EnumMeta(type):
         """
         if pyver < 3.0:
             # if class_name is unicode, attempt a conversion to ASCII
-            if isinstance(class_name, unicode):
+            if isinstance(class_name, six.string_types):
                 try:
                     class_name = class_name.encode('ascii')
                 except UnicodeEncodeError:
@@ -440,22 +442,22 @@ class EnumMeta(type):
         __order__ = []
 
         # special processing needed for names?
-        if isinstance(names, basestring):
+        if isinstance(names, six.string_types):
             names = names.replace(',', ' ').split()
-        if isinstance(names, (tuple, list)) and isinstance(names[0], basestring):
+        if isinstance(names, (tuple, list)) and isinstance(names[0], six.string_types):
             names = [(e, i + start) for (i, e) in enumerate(names)]
 
         # Here, names is either an iterable of (name, value) or a mapping.
         item = None  # in case names is empty
         for item in names:
-            if isinstance(item, basestring):
+            if isinstance(item, six.string_types):
                 member_name, member_value = item, names[item]
             else:
                 member_name, member_value = item
             classdict[member_name] = member_value
             __order__.append(member_name)
         # only set __order__ in classdict if name/value was not from a mapping
-        if not isinstance(item, basestring):
+        if not isinstance(item, six.string_types):
             classdict['__order__'] = ' '.join(__order__)
         enum_class = metacls.__new__(metacls, class_name, bases, classdict)
 
@@ -648,7 +650,7 @@ def __new__(cls, value):
             return cls._value2member_map_[value]
     except TypeError:
         # not there, now do long search -- O(n) behavior
-        for member in cls._member_map_.values():
+        for member in list(cls._member_map_.values()):
             if member.value == value:
                 return member
     raise ValueError("%s is not a valid %s" % (value, cls.__name__))
@@ -843,7 +845,7 @@ def _convert(cls, name, module, filter, source=None):
         source = vars(source)
     else:
         source = module_globals
-    members = dict((name, value) for name, value in source.items() if filter(name))
+    members = dict((name, value) for name, value in list(source.items()) if list(filter(name)))
     cls = cls(name, members, module=module)
     cls.__reduce_ex__ = _reduce_ex_by_name
     module_globals.update(cls.__members__)
@@ -872,7 +874,7 @@ def _reduce_ex_by_name(self, proto):
 def unique(enumeration):
     """Class decorator that ensures only unique members exist in an enumeration."""
     duplicates = []
-    for name, member in enumeration.__members__.items():
+    for name, member in list(enumeration.__members__.items()):
         if name != member.name:
             duplicates.append((name, member.name))
     if duplicates:
